/**
 *	@file client_protocol.h
 *
 *	@project btChat
 *
 *	@author Collin R. Mulliner <collin(AT)betaversion.net>
 *
 * (c) Collin R. Mulliner
 *
 */

#ifndef __CLIENT_PROTOCOL_H__
#define __CLIENT_PROTOCOL_H__

enum btcclient_protocol_pkgtype_t {
	BTCCLIENT_PROTOCOL_NULL = 0,
	BTCCLIENT_PROTOCOL_ACCOUNT,
	BTCCLIENT_PROTOCOL_MSG,
	BTCCLIENT_PROTOCOL_UNKNOWN = 255
};

/**
 *	@brief protocol header
 *
 *	every protocol event beginns with this header
 *
 */
struct btcclient_protocol_header_t {
	__uint8_t version;	/**< protocol version */
	__uint8_t pkg_type;	/**< packet type - see btcclient_protocol_pkgtype_t */
	__uint32_t time;		/**< time at which packet was send (un*x time in seconds) */
	__uint16_t length;	/**< packet length, inclusive header */
};

/**
 *	@brief an account/device/node
 *
 */
struct btcclient_protocol_account_t {
	__uint8_t addr[20];	/**< device bdaddr */
	__uint32_t device_class;	/**< bluetooth device class */
	__uint8_t name[250];	/**< bluetooth device name */
	__uint8_t btchat_channel;	/**< btchat rfcomm channel */
	__uint8_t discovery_method;	/**< method of device discovery */
	__int8_t trust_level;	/**< trust level */
	__uint8_t connection;	/**< type of connection */
	__uint8_t state;	/**< general state */
};

struct btcchlient_protocol_msg_t {
	__uint8_t src[6];	/**< source bdaddr */
	__uint8_t dst[6];	/**< destination bdaddr */
	
	/* __uint8_t *msg; */	/**< message bytes */
};

struct btcchlient_protocol_trustlevel_t {
	__uint8_t src[6];	/**< bdaddr */
	__uint16_t reserved;	/**< reserved */
	__uint32_t level;	/**< trust level */
};

#endif
