/**
 *	@file main.c
 *
 *	@project btChat
 *
 *	@author Collin R. Mulliner <collin(AT)betaversion.net>
 *
 * (c) Collin R. Mulliner
 *
 */

#define _GNU_SOURCE
#include <assert.h>
#include <errno.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <termios.h>
#include <time.h>
#include <unistd.h>
#include <dirent.h>
#include <netinet/in.h>
#include <netdb.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>
#include <bluetooth/sdp.h>
#include <bluetooth/sdp_lib.h>
#include <bluetooth/rfcomm.h>

#include "tree.h"
#include "bt_discover.h"
#include "btchatd.h"
#include "event.h"
#include "net.h"
#include "bt.h"
#include "util.h"

#define BTCHAT_INFO_TEXT ""\
"Welcome to the first alpha test of btChat!\n\n"\
"Currently btChat only monitors your Bluetooth(tm) environment for you, this\n"\
"means you will see each Bluetooth(tm) device as a entry in your buddy list.\n\n"\
"GAIM setup:\n"\
"\tScreen Name:\tanything\n"\
"\tPassword:\tanything\n"\
"\tProxy type:\tNo Proxy\n"\
"\n"\
"btChatd version: 0.0.2\n"\
"contact: collin-btchat@betaversion.net\n"\
"web:     http://www.mulliner.org/bluetooth/btchat/\n\n"\
"NOW connect GAIM\n"

struct btchatd_t *globals = NULL;

void signal_term(int signum)
{
	if (globals->net.server != -1) close(globals->net.server);
	if (globals->net.client != -1) close(globals->net.client);
}

int main(int argc, char **argv)
{
	int commsockets[2];
	int childpid;
	/*struct btchat_event_t *e;*/
	int i;


	printf(BTCHAT_INFO_TEXT);

	signal(SIGTERM, signal_term);		
	
	/* bt-discover server */
	if (socketpair(AF_LOCAL, SOCK_STREAM, 0, commsockets) != 0) {
		#ifdef DEBUG
		perror("main(): socketpair");
		#endif
		return(-1);
	}
	
	/* fork of discover process */
	childpid = fork();
	if (childpid == -1) {
		#ifdef DEBUG
		perror("main(): fork");
		#endif
		return(-1);
	}
	/* this is the child */
	else if (childpid == 0) {
		close(commsockets[1]);
		bt_discover_server2(commsockets[0]);
		return(1);
	}
	/* this is the main process */
	else {
		close(commsockets[0]);
		
		globals = (struct btchatd_t*)	malloc(sizeof(struct btchatd_t));
	
		assert(globals != NULL);
	
		memset(globals, 0, sizeof(struct btchatd_t));
		
		/* the bt-discover client */
		globals->btdiscover.s = commsockets[1];
		globals->btdiscover.pid = childpid;
		globals->btdiscover.inquiry_duration = 10;
		globals->btdiscover.buffer_pos = 0;
		globals->btdiscover.buffer_size = 0xffff;
		globals->btdiscover.local_monitor = 1;
		globals->btdiscover.local_monitor_interval = 60;
	
		/* network setup */
		globals->net.port = 2222;
		strncpy(globals->net.local_ip, "127.0.0.1\0", 256);
		globals->net.server = -1;
		globals->net.client = -1;
		globals->net.state = 0;
		globals->net.buffer_pos = 0;
		globals->net.buffer_size = 0xffff;
	
		/* bluetooth setup */
		globals->btserver.server = -1;
		globals->btserver.state = 0;
		globals->btserver.channel = 1;
		bacpy(&globals->btserver.local_bdaddr, BDADDR_ANY);
		/* init direct connections */
		for (i = 0; i < 7; i++) {
			globals->btserver.clients[i].s = -1;
			globals->btserver.clients[i].buffer_pos = 0;
			globals->btserver.clients[i].buffer_size = 0xffff;
		}
		
		if (net_server(globals, NULL) != 1) {
			#ifdef DEBUG
			printf("main(): net_server failed\n");
			#endif
			return(-1);
		}
		
		if (bt_server(globals, NULL) != 1) {
			#ifdef DEBUG
			printf("main(): bt_server failed\n");
			#endif
			return(-1);
		}

		/* node tree */
		globals->nodes.num = 0;
		globals->nodes.root = NULL;
		globals->nodes.compare_function = btchat_tree_cmp;
		globals->nodes.walk_function = NULL;
		
		/* general setup */
		globals->running = 1;
		
		/* "cron" jobs */
		globals->tasks.interval = 60;	/* seconds */
		globals->tasks.last = 0;
		globals->tasks.next = 0;
		
		/* test events */
	/*	e = malloc(sizeof(struct btchat_event_t));
		memset(e, 0, sizeof(struct btchat_event_t));
		e->etype = BTCHAT_EVENT_DISCOVER;
		event_put(&globals->btdiscover.events, e);
	*/
		
		/* application main loop */
		return(event_loop(globals));
	}
	
	return(0);
}
